%% Plotting the output spectrum of the KD1S and SNR
%%
clear all; close all; clc;

%% Process data
VDD = 5;
VCM = VDD/2;
Kpath = 8;
OSR_init = 64;  % Kpath*OSR
tstart = 100e-9; % time simulation starts saving data

% IMPORTANT !!! Make sure that the result directory path is correct in your setup !!!
rdir = '/home/faculty/jbaker/cadence/simulation/_Fig_9_33_Matlab_MSD/spectre/schematic/psf';


% Pick transient data-set
dataset_name = 'tran-tran';

% signals = cds_srr(rdir, dataset_name); % List all signals

% Extract the voltage information from the signals
Q0 = cds_srr(rdir, dataset_name, 'b0'); q_0 = Q0.V;
Q1 = cds_srr(rdir, dataset_name, 'b1'); q_1 = Q1.V;
Q2 = cds_srr(rdir, dataset_name, 'b2'); q_2 = Q2.V;
Q3 = cds_srr(rdir, dataset_name, 'b3'); q_3 = Q3.V;
Q4 = cds_srr(rdir, dataset_name, 'b4'); q_4 = Q4.V;
Q5 = cds_srr(rdir, dataset_name, 'b5'); q_5 = Q5.V;
Q6 = cds_srr(rdir, dataset_name, 'b6'); q_6 = Q6.V;
Q7 = cds_srr(rdir, dataset_name, 'b7'); q_7 = Q7.V;
CLK = cds_srr(rdir, dataset_name, 'clk'); clk = CLK.V;

% Extract input signal
VIN = cds_srr(rdir, dataset_name, 'Vin'); 
VIN1 = cds_evalsig(VIN, strcat('time >=', num2str(tstart))); % Discard first tstart of signal
vin = VIN1.V;
tRaw = VIN1.time - tstart;    % Extract time information

tmax = tRaw(end);   % Simulation time

% Convert the digital waveforms into binary values
clk_sliced = clk > VCM;
clktrans = find(clk_sliced(2:end)-clk_sliced(1:end-1) == 1);

b_c(1,:) = (q_0(clktrans)) > VCM;
b_c(2,:) = (q_1(clktrans)) > VCM;
b_c(3,:) = (q_2(clktrans)) > VCM;
b_c(4,:) = (q_3(clktrans)) > VCM;
b_c(5,:) = (q_4(clktrans)) > VCM;
b_c(6,:) = (q_5(clktrans)) > VCM;
b_c(7,:) = (q_6(clktrans)) > VCM;
b_c(8,:) = (q_7(clktrans)) > VCM;
Nc = length(b_c(1,:));


%% Cycle through parallel and serial modes
for modenum = 0:1  %set to 0 for serial, 1 for parallel
   %% Compute time domain signal
   if modenum == 0  %serial
      % for serial signal
      vout = VDD*reshape(b_c, Nc*Kpath, 1); 
      OSR = OSR_init;
      mode = 'Serial';
   else %parallel
      vout = (VDD/Kpath)*sum(b_c, 1)';
      OSR = OSR_init/Kpath;
      mode = 'Parallel';
   end

   N=length(vout);
   vac = vout - mean(vout);

   %% Compute intermediate values
   ts = tmax/(N-1);  %average time step between clocks
   t = 0:ts:tmax;
   fs = 1/ts;        %derived clock (sampling) frequency
   fd = 1/tmax;      %frequency steps in the fft
   fmax = fs/2;      %Nyquist frequency

   %% Apply Hann Window and compute spectrum
   w = hann(N);
   w1 = norm(w,1);
   Y = fft(w.*vac)/(w1); %Apply window and estimate FFT
   N1 = floor(N/2);  %lower half of fft indices
   Yl = Y(1:N1); %lower half of fft

   %% Estimate PSD
   Py = abs(Yl).^2; %Power spectrum
   %remove residual DC component and effects due to windowing
   Py(1) = min(Py(1:20));  % Eliminate DC component; use minimum instead of zero so log stays in range
   Py(2) = Py(1);  % Eliminate windowing component due to any residual DC in time domain
   [Pmax, fbin] = max(Py); %The tone power and index
   f = 0:fd:(N1-1)*fd;  %freq array

   %% Estimate SNR and Neff per Schreier's Appendix A
   nb = 5;
   signal_bins = fbin + (-(nb-1)/2:(nb-1)/2);
   inband_bins = 1:N/(2*OSR);
   noise_bins = setdiff(inband_bins,signal_bins);
   % Remove distortion tones
   second_tone_bins = 2*fbin-1 + (-(nb-1)/2:(nb-1)/2);
   noise_bins = setdiff(noise_bins,second_tone_bins);
   third_tone_bins = 3*fbin-1 + (-(nb-1)/2:(nb-1)/2);
   noise_bins = setdiff(noise_bins,third_tone_bins);
   % Calculate SNR (leave distortion terms in for SNDR)
   other_bins = max(noise_bins):N1;
   SNR = 10*log10(sum(Py(signal_bins))/sum(Py(noise_bins)));
   Neff = (SNR-1.78)/6.02;

   %% Plot Data
   scrsz = get(0,'ScreenSize');
   w=.2;

   figure('Position',[1+modenum*1.1*w*scrsz(3) (1-.3*(1.05*1))*scrsz(4) w*scrsz(3) .25*scrsz(4)])
   t1 = reshape([t; [t(2:end) 2*t(end)-t(end-1)]], 1, 2*length(t));
   v1 = reshape([vac'+VDD/2; vac'+VDD/2], 1, 2*length(vac));
   plot(t1, v1);
   %    plot(t, vac+VDD/2);
   hold on; plot(tRaw, vin, 'r'); hold off;
   xlabel ('Time'); ylabel ('Volts');
   title (['KD1S Output in ' mode ' mode']);
   legend ([{'Out'}; {'In'}]);
   grid on;

   figure('Position',[1+modenum*1.1*w*scrsz(3) (1-.3*(1.05*2))*scrsz(4) w*scrsz(3) .25*scrsz(4)])
   semilogx(f(signal_bins), 10*log10(Py(signal_bins)), 'g', 'linewidth', 2)  %plot signal spectrum
   hold on
   semilogx(f(noise_bins),  10*log10(Py(noise_bins)), 'or');
   semilogx(f(other_bins),  10*log10(Py(other_bins)), 'b');
   hold off
   xlabel ('Frequency'); ylabel ('dB');
   title (['KD1S Output Spectrum in ' mode ' mode, SNR = ' num2str(SNR) 'dB']);
   legend ([{'Signal'}; {'Noise'}; {'> B'}]);
   grid on;

   figure('Position',[1+modenum*1.1*w*scrsz(3) (1-.3*(1.05*3))*scrsz(4) w*scrsz(3) .25*scrsz(4)])
   otherrange = .5;  %portion of the plot devoted to "other" frequencies, use 1 for all
   Nother = min(N1, floor(max(noise_bins)/(1-otherrange))) - max(noise_bins);
   plot(f(signal_bins), 10*log10(Py(signal_bins)), 'g', 'linewidth', 2);  %plot signal spectrum
   hold on
   plot(f(noise_bins),  10*log10(Py(noise_bins)), 'or');
   plot(f(other_bins(1:Nother)),  10*log10(Py(other_bins(1:Nother))), 'b');
   hold off
   xlabel ('Frequency'); ylabel ('dB');
   title (['KD1S Output Spectrum in ' mode ' mode, SNR = ' num2str(SNR) 'dB']);
   legend ([{'Signal'}; {'Noise'}; {'> B'}]);
   grid on;

   %% Send summary SNR data to command window
   if modenum == 0  %serial
      fprintf(1, '\nKpath = %.0f OSR = %.0f', Kpath, OSR)
      fprintf(1, '\nFor 1-bit output (serial mode) at Kpath*fs = %.0f MHz', fs/1e6)
      fprintf(1, '\nSNR = %.2f, Neff = %.2f, B = %.2f MHz', SNR, Neff, fs/(2*OSR*1e6))
   else %parallel
      fprintf(1, '\nFor Kpath-bits output (parallel mode) at fs = %.0f MHz', fs/1e6)
      fprintf(1, '\nSNR = %.2f, Neff = %.2f, B = %.2f MHz', SNR, Neff, fs/(2*OSR*1e6))
   end

   %% To plot reconstructed signal pass data through two Sinc Filters
   B=ones(1,OSR)/OSR;A=1;
   voutf1 = filter(B,A,vout);
   voutf2 = filter(B,A,voutf1);
   vout_recon = voutf2(2*OSR:end); %remove filter start-up transient
   figure('Position',[1+(modenum+2)*1.1*w*scrsz(3) (1-.3*(1.05*1))*scrsz(4) w*scrsz(3) .25*scrsz(4)])
   tnew = t(2*OSR:end);
   tnew1 = reshape([tnew; [tnew(2:end) 2*tnew(end)-t(end-1)]], 1, 2*length(tnew));
   vout_recon1 = reshape([vout_recon'; vout_recon'], 1, 2*length(vout_recon));
   plot(tnew1, vout_recon1);
   %    plot(tnew, vout_recon);
   hold on; plot(tRaw, vin, 'r'); hold off;
   xlabel ('Time'); ylabel ('Volts');
   title (['Filtered KD1S Output in ' mode ' mode']);
   legend ([{'Out'}; {'In'}]);
   grid on;

end



